package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import com.rjconsultores.ventaboletos.entidad.GrupoCategoria;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderGrupoCategoria;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

@Controller("busquedaGrupoCategoriaController")
@Scope("prototype")
public class BusquedaGrupoCategoriaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<GrupoCategoria> plwGrupoCategoria;
	private MyListbox grupoCategoriaList;
	private Paging pagingGrupoCategoria;
	private Textbox txtDesc;

	public Paging getPagingGrupoCategoria() {
		return pagingGrupoCategoria;
	}

	public void setPagingGrupoCategoria(Paging pagingGrupoCategoria) {
		this.pagingGrupoCategoria = pagingGrupoCategoria;
	}

	public MyListbox getGrupoCategoriaList() {
		return grupoCategoriaList;
	}

	public void setGrupoCategoriaList(MyListbox grupoCategoriaList) {
		this.grupoCategoriaList = grupoCategoriaList;
	}

	public PagedListWrapper<GrupoCategoria> getPlwGrupoCategoria() {
		return plwGrupoCategoria;
	}

	public void setPlwGrupoCategoria(PagedListWrapper<GrupoCategoria> plwGrupoCategoria) {
		this.plwGrupoCategoria = plwGrupoCategoria;
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verPeriodo(new GrupoCategoria());
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		grupoCategoriaList.setItemRenderer(new RenderGrupoCategoria());
		grupoCategoriaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				GrupoCategoria c = (GrupoCategoria) grupoCategoriaList.getSelected();
				verPeriodo(c);
			}
		});

		refreshLista();

		txtDesc.focus();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verPeriodo(GrupoCategoria c) {
		if (c == null) {
			return;
		}

		Map args = new HashMap();
		args.put("grupoCategoria", c);
		args.put("grupoCategoriaList", grupoCategoriaList);
		args.put("busquedaGrupoCategoriaController", this);

		openWindow("/gui/catalogos/editarGrupoCategoria.zul",
				Labels.getLabel("editarGrupoCategoriaController.window.title"), args, MODAL);
	}

	public void refreshLista() {
		HibernateSearchObject<GrupoCategoria> claseServicioBusqueda =
				new HibernateSearchObject<GrupoCategoria>(GrupoCategoria.class, pagingGrupoCategoria.getPageSize());

		claseServicioBusqueda.addFilterLike("descGrupo", "%" + txtDesc.getText().trim().concat("%"));
		claseServicioBusqueda.addFilterEqual("activo", Boolean.TRUE);
		claseServicioBusqueda.addSortAsc("descGrupo");

		plwGrupoCategoria.init(claseServicioBusqueda, grupoCategoriaList, pagingGrupoCategoria);
		if (grupoCategoriaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaGrupoCategoriaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}
}
