package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.ComboitemRenderer;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Radio;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.TipoTarifaPacote;
import com.rjconsultores.ventaboletos.entidad.Usuario;
import com.rjconsultores.ventaboletos.relatorios.impl.RelatorioVendasPacotesBoletos;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.TipoTarifaPacoteService;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxParada;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxParadaCve;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxUsuario;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;

import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.util.JRLoader;

@Controller("relatorioVendasPacotesBoletosController")
@Scope("prototype")
public class RelatorioVendasPacotesBoletosController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	
	@Autowired
	private DataSource dataSourceRead;
	
	@Autowired
	private EmpresaService empresaService;
	
	@Autowired
	private TipoTarifaPacoteService tipoTarifaPacoteService; 

	private List<Empresa> lsEmpresa;
	private List<TipoTarifaPacote> lsTipoTarifaPacote;
	private Datebox dataVendaInicial;
	private Datebox dataVendaFinal;
	private Datebox dataPacoteInicial;
	private Datebox dataPacoteFinal;
	private MyComboboxEstandar cmbEmpresa;
	private MyComboboxEstandar cmbTipoTarifaPacote;
	
	private MyComboboxParada cmbParadaOrigem;
	private MyComboboxParadaCve cmbParadaOrigemCve;
	private MyComboboxParada cmbParadaDestino;
	private MyComboboxParadaCve cmbParadaDestinoCve;
	private MyComboboxUsuario cmbUsuario;
	private Radio rTodos;
	private Radio rVendidos;
	private Radio rCancelados;

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodosIncluindoEmpresaTodas();
		
		lsTipoTarifaPacote = new ArrayList<TipoTarifaPacote>();
		lsTipoTarifaPacote.add(new TipoTarifaPacote(-1, "TODAS"));
		lsTipoTarifaPacote.addAll(tipoTarifaPacoteService.obtenerTodos());
		super.doAfterCompose(comp);
		
		cmbParadaOrigemCve.setItemRenderer(new ComboitemRenderer() {
			@Override
			public void render(Comboitem cmbtm, Object o) throws Exception {
				Parada parada = (Parada) o;

				cmbtm.setLabel(parada.getCveparada());
				cmbtm.setValue(parada);
			}
		});

		cmbParadaDestinoCve.setItemRenderer(new ComboitemRenderer() {
			@Override
			public void render(Comboitem cmbtm, Object o) throws Exception {
				Parada parada = (Parada) o;

				cmbtm.setLabel(parada.getCveparada());
				cmbtm.setValue(parada);
			}
		});
		
		cmbUsuario.setItemRenderer(new ComboitemRenderer() {
			@Override
			public void render(Comboitem cmbtm, Object o) throws Exception {
				Usuario usuario = (Usuario) o;
				
				cmbtm.setLabel(usuario.getNombusuario());
				cmbtm.setValue(usuario);
			}
		});

	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		Date dataVendaDe = dataVendaInicial.getValue();
		Date dataVendaAte = dataVendaFinal.getValue();
		Date dataPacoteDe = dataPacoteInicial.getValue();
		Date dataPacoteAte = dataPacoteFinal.getValue();

		Map<String, Object> parametros = new HashMap<String, Object>();
		String periodoVenda = "";
		String periodoPacote = "";
		if(dataVendaDe != null) {
			parametros.put("fecVendaInicio", sdf.format(dataVendaDe));
			periodoVenda += parametros.get("fecVendaInicio") + " ";
		}
		if(dataVendaAte != null) {
			parametros.put("fecVendaFinal", sdf.format(dataVendaAte));
			periodoVenda += Labels.getLabel("lb.ate") + " " + parametros.get("fecVendaFinal");
		}
		if(dataPacoteDe != null) {
			parametros.put("fecPacoteInicio", sdf.format(dataPacoteDe));
			periodoPacote += parametros.get("fecPacoteInicio") + " ";
		}
		if(dataPacoteAte != null) {
			parametros.put("fecPacoteFinal", sdf.format(dataPacoteAte));
			periodoPacote += Labels.getLabel("lb.ate") + " " + parametros.get("fecPacoteFinal");
		}
		
		parametros.put("periodoVenda", StringUtils.isNotBlank(periodoVenda) ? periodoVenda : "Todos");
		parametros.put("periodoPacote", StringUtils.isNotBlank(periodoPacote) ? periodoPacote : "Todos");

		Comboitem cbiEmpresa = cmbEmpresa.getSelectedItem();
		String empresaId = null;
		parametros.put("empresa", "");
		if (cbiEmpresa != null) {
			Empresa empresa = (Empresa) cbiEmpresa.getValue();
			empresaId = empresa.getEmpresaId().toString();
			parametros.put("empresa", empresa.getNombempresa());
		}
		parametros.put("empresaId", empresaId);
		
		parametros.put("origem","Todas");
		Comboitem cbiOrigem = cmbParadaOrigem.getSelectedItem();
		if (cbiOrigem != null) {
			Parada origem = (Parada) cbiOrigem.getValue();
			parametros.put("origenId", origem.getParadaId());
			parametros.put("origem", origem.getDescparada());
		}

		parametros.put("destino","Todos");
		Comboitem cbiDestino = cmbParadaDestino.getSelectedItem();
		if (cbiDestino != null) {
			Parada destino = (Parada) cbiDestino.getValue();
			parametros.put("destinoId", destino.getParadaId());
			parametros.put("destino", destino.getDescparada());
		}

		parametros.put("usuario","Todos");
		Comboitem cbiUsuario = cmbUsuario.getSelectedItem();
		if (cbiUsuario != null) {
			Usuario usuario = (Usuario) cbiUsuario.getValue();
			parametros.put("usuarioId", usuario.getUsuarioId());
			parametros.put("usuario", usuario.getNombUsuarioCompleto());
		}
		
		parametros.put("tipoTarifaPacote","Todas");
		Comboitem cbiTipoTarifaPacote = cmbTipoTarifaPacote.getSelectedItem();
		if (cbiTipoTarifaPacote != null) {
			TipoTarifaPacote tipoTarifaPacote = (TipoTarifaPacote) cbiTipoTarifaPacote.getValue();
			parametros.put("tipoTarifaPacoteId", tipoTarifaPacote.getTipotarifapacoteId());
			parametros.put("tipoTarifaPacote", tipoTarifaPacote.getDesctipotarifa());
		}
		
		parametros.put("situacaoBilhete", "Todos");
		if(rVendidos.isChecked()) {
			parametros.put("situacaoBilhete", "Vendidos");
			parametros.put("situacaoBilheteId", RelatorioVendasPacotesBoletos.VENDIDOS);
		} else if(rCancelados.isChecked()) {
			parametros.put("situacaoBilhete", "Cancelados");
			parametros.put("situacaoBilheteId", RelatorioVendasPacotesBoletos.CANCELADOS);
		}
		
		Relatorio relatorio = gerarRelatorioVendaPacotesBoletos(parametros);
		
		Map<String, Object> args = new HashMap<String, Object>();
		args.put("relatorio", relatorio);

		openWindow("/component/reportView.zul",
				   Labels.getLabel("relatorioVendasPacotesBoletosController.window.title"), args, MODAL);
	}

	private Relatorio gerarRelatorioVendaPacotesBoletos(Map<String, Object> parametros) throws SQLException, Exception {
		JasperReport subRelatorioVendasPacotesBoletosItens = (JasperReport) JRLoader.loadObject(this.getClass().getResourceAsStream("/com/rjconsultores/ventaboletos/relatorios/templates/RelatorioVendasPacotesBoletosItem.jasper"));
		parametros.put("subreporte", subRelatorioVendasPacotesBoletosItens);

		JasperReport subRelatorioVendasPacotesBoletosItensTotais = (JasperReport) JRLoader.loadObject(this.getClass().getResourceAsStream("/com/rjconsultores/ventaboletos/relatorios/templates/RelatorioVendasPacotesBoletosItemTotais.jasper"));
		parametros.put("subreporteTotais", subRelatorioVendasPacotesBoletosItensTotais);
		
		return new RelatorioVendasPacotesBoletos(parametros, dataSourceRead.getConnection(), "RelatorioVendasPacotesBoletosItem", "RelatorioVendasPacotesBoletosItemTotais");
	}
	
	public void onSelect$cmbParadaOrigemCve(Event ev) {
		if (cmbParadaOrigemCve.getSelectedItem() != null) {
			cmbParadaOrigem.setComboItemByParada((Parada) cmbParadaOrigemCve.getSelectedItem().getValue());
		}
	}

	public void onSelect$cmbParadaOrigem(Event ev) {
		if (cmbParadaOrigem.getSelectedItem() != null)
			cmbParadaOrigemCve.setComboItemByParada((Parada) cmbParadaOrigem.getSelectedItem().getValue());
	}

	public void onSelect$cmbParadaDestinoCve(Event ev) {
		if (cmbParadaDestinoCve.getSelectedItem() != null) {
			cmbParadaDestino.setComboItemByParada((Parada) cmbParadaDestinoCve.getSelectedItem().getValue());
		}
	}

	public void onSelect$cmbParadaDestino(Event ev) {
		if (cmbParadaDestino.getSelectedItem() != null) {
			cmbParadaDestinoCve.setComboItemByParada((Parada) cmbParadaDestino.getSelectedItem().getValue());
		}
	}

	public List<TipoTarifaPacote> getLsTipoTarifaPacote() {
		return lsTipoTarifaPacote;
	}

	public void setLsTipoTarifaPacote(List<TipoTarifaPacote> lsTipoTarifaPacote) {
		this.lsTipoTarifaPacote = lsTipoTarifaPacote;
	}

}
